@@ -214,15 +214,20 @@ class AlignedDataPair(object):
         return pd.Series(obs1.index.tolist() + obs2.index.tolist())
 
     @property
-    def _varnames_node(self, ):
+    def varnames_node(self, ):
         """ Names of variable nodes """
         return self.var['name'].values.copy()
 
+    @property
+    def varnames_feat(self, ):
+        """ The observation feature names """
+        return self._varnames_feat.values.copy()
+
     def __str__(self):
         s = "\n".join([
-            f"DataPair with {self.n_obs} obs- and {self.n_vnodes} var-nodes",
+            f"AlignedDataPair with {self.n_obs} obs- and {self.n_vnodes} var-nodes",
             f"n_obs1 ({self.dataset_names[0]}): {self.n_obs1}",
-            f"n_obs2 ({self.dataset_names[0]}): {self.n_obs2}",
+            f"n_obs2 ({self.dataset_names[1]}): {self.n_obs2}",
             f"Dimensions of the obs-node-features: {self.n_feats}",
         ])
         return s
@@ -241,8 +246,8 @@ class AlignedDataPair(object):
             features=self._features,
             ov_adjs=self._ov_adjs,
             oo_adjs=self._oo_adj,
-            varnames_feat=self._varnames_feat,
-            varnames_node=self._varnames_node,
+            varnames_feat=self.varnames_feat,
+            varnames_node=self.varnames_node,
             obs_dfs=self.obs_dfs,
             dataset_names=self.dataset_names,
             ntypes=self.ntypes,
@@ -252,16 +257,24 @@ class AlignedDataPair(object):
         logging.info(f"inputs for construction (aligned) datapair saved into {path}")
 
     def get_obs_features(self, astensor=True, scale=True,
-                         unit_var=True,
+                         unit_var=True, batch_keys=None,
                          clip=False, clip_range=(-3, 3.5)):
-        feats = self._features
+        features = self._features
         if scale:
-            def zscore(X, with_mean=True,):
-                return utp.zscore(X, with_mean=with_mean, scale=unit_var)
+            feats = []
+            if batch_keys is None:
+                batch_keys = [None, None]
+            for X, _df, bch_key in zip(features, self.obs_dfs, batch_keys):
+                if bch_key is None:
+                    feats.append(utp.zscore(X, with_mean=True, scale=unit_var))
+                else:
+                    bch_lbs = _df[bch_key]
+                    feats.append(
+                        utp.group_zscore(X, bch_lbs, with_mean=True, scale=unit_var)
+                    )
+            features = feats
 
-            feats = list(map(zscore, feats))
-
-        features = np.vstack(feats)
+        features = np.vstack(features)
         if clip:
             vmin, vmax = clip_range
             print(f'clipping feature values within [{vmin: .2f}, {vmax: .2f}]')
@@ -479,8 +492,8 @@ class AlignedDataPair(object):
         """
         if len(ov_adjs) == 2:
             print('[*] Setting observation-by-variable adjacent matrices '
-                  '(`self._ov_adjs`) for making merged graph adjacent matrix of '
-                  'observation and variable nodes')
+                  '(`self._ov_adjs`) for making merged graph adjacent matrix of'
+                  ' observation and variable nodes')
             n_obs1, n_vnodes1 = ov_adjs[0].shape
             n_obs2, n_vnodes2 = ov_adjs[1].shape
             # checking dimensions
@@ -510,7 +523,7 @@ class AlignedDataPair(object):
     def set_varnames_node(self, varnames_node=None, index=None):
         if varnames_node is None:
             varnames_node = list(range(self.n_vnodes))
-        #        self._varnames_node = pd.Series(varnames_node)
+        #        self.varnames_node = pd.Series(varnames_node)
         if not hasattr(self, 'var') or self.var is None:
             self.var = pd.DataFrame({'name': varnames_node}, index=index)
         else:
@@ -701,15 +714,15 @@ def aligned_datapair_from_adatas(
     obs2 = adata2.obs.copy()
 
     # --- node features (for single-cells)
-    # try:
-    #     features1 = adata1[:, vars_feat].X
-    #     features2 = adata2[:, vars_feat].X
-    # except:
-    logging.info(
-        '[NOTE]\nthe node features will be extracted from `adata.raw`, '
-        'please make sure that the values are normalized.\n')
-    features1 = adata_raw1[:, vars_feat].X
-    features2 = adata_raw2[:, vars_feat].X
+    try:
+        features1 = adata1[:, vars_feat].X
+        features2 = adata2[:, vars_feat].X
+    except:
+        logging.warning(
+            '[NOTE]\nthe node features will be extracted from `adata.raw`, '
+            'please make sure that the values are normalized.\n')
+        features1 = adata_raw1[:, vars_feat].X
+        features2 = adata_raw2[:, vars_feat].X
 
     features = list(map(_check_sparse_toarray, [features1, features2]))
 
