@@ -233,6 +233,18 @@ class DataPair(object):
         """Names of the variable (e.g., gene) nodes in the query data"""
         return self._var_id2name[self.n_vnodes1:].tolist()
 
+    @property
+    def feat_names1(self, ):
+        """Feature ames of the observation (e.g., cell) nodes
+        in the reference data"""
+        return self._varnames_feat.iloc[:, 0].values
+
+    @property
+    def feat_names2(self, ):
+        """Feature ames of the observation (e.g., cell) nodes
+        in the query data"""
+        return self._varnames_feat.iloc[:, 1].values
+
     @property
     def labels(self, ):
         """ Labels for each observations that would be taken as the supervised
@@ -257,7 +269,7 @@ class DataPair(object):
         s = "\n".join([
             f"DataPair with {self.n_obs} obs- and {self.n_vnodes} var-nodes",
             f"obs1 x var1 ({self.dataset_names[0]}): {self.n_obs1} x {self.n_vnodes1}",
-            f"obs2 x var2 ({self.dataset_names[0]}): {self.n_obs2} x {self.n_vnodes2}",
+            f"obs2 x var2 ({self.dataset_names[1]}): {self.n_obs2} x {self.n_vnodes2}",
             f"Dimensions of the obs-node-features: {self.n_feats}",
         ])
         return s
@@ -289,16 +301,24 @@ class DataPair(object):
 
     # In[]
     def get_obs_features(self, astensor=True, scale=True,
-                         unit_var=True,
+                         unit_var=True, batch_keys=None,
                          clip=False, clip_range=(-3, 3.5)):
-        feats = self._features
+        features = self._features
         if scale:
-            def zscore(X, with_mean=True, ):
-                return utp.zscore(X, with_mean=with_mean, scale=unit_var)
+            feats = []
+            if batch_keys is None:
+                batch_keys = [None, None]
+            for X, _df, bch_key in zip(features, self.obs_dfs, batch_keys):
+                if bch_key is None:
+                    feats.append(utp.zscore(X, with_mean=True, scale=unit_var))
+                else:
+                    bch_lbs = _df[bch_key]
+                    feats.append(
+                        utp.group_zscore(X, bch_lbs, with_mean=True, scale=unit_var)
+                    )
+            features = feats
 
-            feats = list(map(zscore, feats))
-
-        features = np.vstack(feats)
+        features = np.vstack(features)
         if clip:
             vmin, vmax = clip_range
             print(f'clipping feature values within [{vmin: .2f}, {vmax: .2f}]')
@@ -402,7 +422,7 @@ class DataPair(object):
             anno2 = self.obs_dfs[1][keys[1]]
 
         if concat:
-            return np.array(anno1.tolist() + anno2.tolist())
+            return np.array(list(anno1) + list(anno2))
         else:
             return anno1, anno2
 
@@ -668,7 +688,7 @@ class DataPair(object):
                         force_reset=False,
                         **kwannos):
 
-        if self.var is not None or force_reset:
+        if self.var is None or force_reset:
             self.var = self._var_id2name.to_frame(self._KEY_VARNAME)
             dsn_lbs = self.n_vnodes1 * [self.dataset_names[0]] + \
                       self.n_vnodes2 * [self.dataset_names[1]]
@@ -871,22 +891,22 @@ def datapair_from_adatas(
             df_varmap_1v1, vars_all1, vars_all2, union=False)
         submaps_1v1 = utp.subset_matches(
             submaps_1v1, vars_use1, vars_use2, union=True)
-        print("TEST", submaps_1v1.shape)
+        # print("TEST", submaps_1v1.shape)
     else:
-        # (intersection of 1-1 matched freatures)
+        # (intersection of 1-1 matched features)
         submaps_1v1 = utp.subset_matches(
             df_varmap_1v1, vars_use1, vars_use2, union=False)
     vnames_feat1, vnames_feat2 = list(zip(*submaps_1v1.values[:, :2]))
 
-    #    try:
-    #        features1 = adata1[:, vnames_feat1].X
-    #        features2 = adata2[:, vnames_feat2].X
-    #    except:
-    logging.info(
-        '[NOTE]\nthe node features will be extracted from `adata.raw`, '
-        'please make sure that the values are normalized.\n')
-    features1 = adata_raw1[:, vnames_feat1].X
-    features2 = adata_raw2[:, vnames_feat2].X
+    try:
+        features1 = adata1[:, vnames_feat1].X
+        features2 = adata2[:, vnames_feat2].X
+    except:
+        logging.warning(
+            '[NOTE]\nthe node features will be extracted from `adata.raw`, '
+            'please make sure that the values are normalized.\n')
+        features1 = adata_raw1[:, vnames_feat1].X
+        features2 = adata_raw2[:, vnames_feat2].X
 
     features = list(map(_check_sparse_toarray, [features1, features2]))
 
