@@ -38,7 +38,7 @@ from . import (
     CGGCNet, datapair_from_adatas,
     CGCNet, aligned_datapair_from_adatas
 )
-from .utils.train import prepare4train, Trainer, seed_everything
+from .utils.train import prepare4train, Trainer
 
 PARAMS_MODEL = get_model_params()
 PARAMS_PRE = get_preprocess_params()
@@ -558,7 +558,7 @@ def gather_came_results(
     # all hidden states
     from .model import get_all_hidden_states
     hidden_list = get_all_hidden_states(
-        trainer.model, trainer.feat_dict, trainer.g)
+        trainer.model, trainer.feat_dict, trainer.g, batch_size=batch_size)
     if save_hidden_list:
         from . import save_hidden_states
         save_hidden_states(hidden_list, resdir / 'hidden_list.h5')
@@ -572,7 +572,7 @@ def gather_came_results(
     pd.DataFrame(out_cell[dpair.obs_ids1], columns=classes).to_csv(resdir / "df_logits1.csv")
     pd.DataFrame(out_cell[dpair.obs_ids2], columns=classes).to_csv(resdir / "df_logits2.csv")
     predictor = Predictor(classes=classes).fit(
-            out_cell[dpair.obs_ids1],
+            out_cell[trainer.train_idx],
             detach2numpy(trainer.train_labels),
         )
     predictor.save(resdir / f'predictor.json')
@@ -581,7 +581,7 @@ def gather_came_results(
     cl_preds = predict_from_logits(probas_all, classes=classes)
     obs = pd.DataFrame(
         # TODO: `keys[0]` -> 'REF', may raise another exceptions
-        {'REF': dpair.get_obs_labels(keys, asint=False),
+        {'REF': dpair.get_obs_labels(keys, asint=False, categories=classes),
          # true labels with `unknown` for unseen classes in query data
          'celltype': dpair.get_obs_anno(keys_compare),  # labels for comparison
          'predicted': cl_preds,
@@ -812,7 +812,7 @@ def preprocess_unaligned(
     node_source = node_source.lower()
     if 'hvg' in node_source and 'deg' in node_source:
         vars_as_nodes = [np.unique(np.hstack([hvgs1, degs1])),
-                        np.unique(np.hstack([hvgs2, degs2]))]
+                         np.unique(np.hstack([hvgs2, degs2]))]
     elif 'hvg' in node_source:
         vars_as_nodes = [hvgs1, hvgs2]
     else:
@@ -827,7 +827,8 @@ def preprocess_unaligned(
     return dct, (adata1, adata2)
 
 
-def __test1__(n_epochs: int = 5, batch_size=None):
+def __test1__(n_epochs: int = 5, batch_size=None, reverse=False):
+    from .utils.train import seed_everything
     seed_everything()
     datadir = Path(os.path.abspath(__file__)).parent / 'sample_data'
     if not datadir.exists():
@@ -836,6 +837,9 @@ def __test1__(n_epochs: int = 5, batch_size=None):
 
     sp1, sp2 = ('human', 'mouse')
     dsnames = ('Baron_human', 'Baron_mouse')
+    if reverse:
+        sp1, sp2 = sp2, sp1
+        dsnames = dsnames[::-1]
 
     df_varmap_1v1 = pd.read_csv(datadir / f'gene_matches_1v1_{sp1}2{sp2}.csv', )
 
@@ -877,7 +881,8 @@ def __test1__(n_epochs: int = 5, batch_size=None):
     print('Test passed for ALIGNED!')
 
 
-def __test2__(n_epochs: int = 5, batch_size=None):
+def __test2__(n_epochs: int = 5, batch_size=None, reverse=False):
+    from .utils.train import seed_everything
     seed_everything()
     datadir = Path(os.path.abspath(__file__)).parent / 'sample_data'
     if not datadir.exists():
@@ -886,6 +891,9 @@ def __test2__(n_epochs: int = 5, batch_size=None):
 
     sp1, sp2 = ('human', 'mouse')
     dsnames = ('Baron_human', 'Baron_mouse')
+    if reverse:
+        sp1, sp2 = sp2, sp1
+        dsnames = dsnames[::-1]
 
     df_varmap_1v1 = pd.read_csv(datadir / f'gene_matches_1v1_{sp1}2{sp2}.csv', )
     df_varmap = pd.read_csv(datadir / f'gene_matches_{sp1}2{sp2}.csv', )
